@@ -49,88 +49,61 @@
 
 package com.openexchange.authentication.database.osgi;
 
-import java.util.concurrent.locks.Lock;
-import java.util.concurrent.locks.ReentrantLock;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import com.openexchange.authentication.AuthenticationService;
+import com.openexchange.authentication.BasicAuthenticationService;
 import com.openexchange.authentication.database.impl.DatabaseAuthentication;
-import com.openexchange.context.ContextService;
-import com.openexchange.user.UserService;
 
 /**
  * Dependently registers the AuthenticationService.
+ *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
-public final class AuthenticationRegisterer implements ServiceTrackerCustomizer<Object,Object> {
+public final class AuthenticationRegisterer implements ServiceTrackerCustomizer<BasicAuthenticationService,BasicAuthenticationService> {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(AuthenticationRegisterer.class);
 
     private final BundleContext context;
-    private final Lock lock = new ReentrantLock();
 
-    private ServiceRegistration<AuthenticationService> registration;
-    private ContextService contextService;
-    private UserService userService;
+    private volatile ServiceRegistration<AuthenticationService> registration;
 
-    public AuthenticationRegisterer(final BundleContext context) {
+    /**
+     * Initializes a new {@link AuthenticationRegisterer}.
+     *
+     * @param context The bundle context
+     */
+    public AuthenticationRegisterer(BundleContext context) {
         super();
         this.context = context;
     }
 
     @Override
-    public Object addingService(final ServiceReference<Object> reference) {
-        final Object obj = context.getService(reference);
-        final boolean needsRegistration;
-        lock.lock();
-        try {
-            if (obj instanceof ContextService) {
-                contextService = (ContextService) obj;
-            }
-            if (obj instanceof UserService) {
-                userService = (UserService) obj;
-            }
-            needsRegistration = null != contextService && null != userService && registration == null;
-        } finally {
-            lock.unlock();
-        }
-        if (needsRegistration) {
-            LOG.info("Registering database authentication service.");
-            registration = context.registerService(AuthenticationService.class,
-                new DatabaseAuthentication(contextService, userService), null);
-        }
-        return obj;
+    public BasicAuthenticationService addingService(ServiceReference<BasicAuthenticationService> reference) {
+        BasicAuthenticationService basicAuthenticationService = context.getService(reference);
+        registration = context.registerService(AuthenticationService.class, new DatabaseAuthentication(basicAuthenticationService), null);
+        LOG.info("Registered database authentication service.");
+        return basicAuthenticationService;
     }
 
     @Override
-    public void modifiedService(final ServiceReference<Object> reference, final Object service) {
+    public void modifiedService(ServiceReference<BasicAuthenticationService> reference, BasicAuthenticationService service) {
         // Nothing to do.
     }
 
     @Override
-    public void removedService(final ServiceReference<Object> reference, final Object service) {
-        ServiceRegistration<?> unregister = null;
-        lock.lock();
+    public void removedService(ServiceReference<BasicAuthenticationService> reference, BasicAuthenticationService service) {
         try {
-            if (service instanceof ContextService) {
-                contextService = null;
-            }
-            if (service instanceof UserService) {
-                userService = null;
-            }
-            if (registration != null && (contextService == null || userService == null)) {
-                unregister = registration;
-                registration = null;
+            ServiceRegistration<AuthenticationService> registration = this.registration;
+            if (null != registration) {
+                registration.unregister();
+                this.registration = null;
             }
         } finally {
-            lock.unlock();
+            context.ungetService(reference);
         }
-        if (null != unregister) {
-            LOG.info("Unregistering database authentication service.");
-            unregister.unregister();
-        }
-        context.ungetService(reference);
     }
+
 }
